; "Bagel", NES chr decompressor.
; Copyright (c) 2016  Johnathan Roatch
;
; Copying and distribution of this file, with or without
; modification, are permitted in any medium without royalty provided
; the copyright notice and this notice are preserved in all source
; code copies.  This file is offered as-is, without any warranty.
;
; Version History:
; 2016-09-19 Initial release, (bytes: 137 ROM)
;

.include "nes.inc"

.importzp temp
.import ppu_buffer

.exportzp bagel_stream_ptr, bagel_block_count
.export bagel_decompress_block
.export bagel_bulk_upload

.segment "ZEROPAGE": zeropage
  bagel_stream_ptr:    .res 2
  bagel_block_count:   .res 1

.segment "CODE"

; bagel_decompress_block is 137 bytes

;;
; bagel_stream_ptr = the input stream pointer,
;     points to after stream on exit.
; X = output buffer offset, restored on exit.
; Returns: flags of decrementing bagel_block_count
.proc bagel_decompress_block
pb8_ctrl = temp+0
plane_counter = temp+1
plane_modes = temp+2
plane_modes_lo = plane_modes+0
plane_modes_hi = plane_modes+1
invert_flag = temp+4
  ldy #$00
  lda (bagel_stream_ptr), y
  iny
  sty plane_counter
  cmp #$c0
  bcc not_short_header
    and #$0f
    sta plane_modes_hi
    asl
    asl
    asl
    asl
    ora plane_modes_hi
    sta plane_modes_hi
  bcc end_read_header  ;,; jmp
  not_short_header:
    sta plane_modes_hi
    lda (bagel_stream_ptr), y
    iny
  end_read_header:
  sta plane_modes_lo
  lda plane_modes_hi
  ; if first plane mode is read previous plane
  ; then skip, effectively duplicating the entire block.
  cmp #$c0
  bcs skip_whole_block
  block_loop:
    lda #$00
    sta invert_flag
    bit plane_modes_hi
    bmi has_pb8_ctrl
      lda #$00
      bvc end_pb8_ctrl_load
        lda #$ff
        clv
    bvc end_pb8_ctrl_load  ;,; jmp
    has_pb8_ctrl:
      lda (bagel_stream_ptr), y
      bne not_skip_plane
        bvs also_not_skip_plane
          txa
          clc
          adc #8
          tax
          iny
          bne next_plane ;,; jmp
        also_not_skip_plane:
      not_skip_plane:
      iny
      cmp #$ff
      bne not_inverted
        sta invert_flag
        lda #$00
      not_inverted:
    end_pb8_ctrl_load:

  pb8_unpack_plane:
    sec
    rol a
    sta pb8_ctrl
    ; for read distance of 1, this is the first byte
    lda invert_flag
    pb8_loop:
      ; at this point:
      ; A: previous byte in this plane
      ; C = 1: copy byte from bitstream
      ; C=0, V=0: repeat previous byte
      ; C=0, V=1: copy byte from previous plane
      bcc use_prev_byte
        lda (bagel_stream_ptr), y
        iny
      bcs byte_done_read
      use_prev_byte:
        bvc not_8_bytes_back
          lda a:ppu_buffer-8, x
          eor invert_flag
        not_8_bytes_back:
      byte_done_read:
      sta ppu_buffer, x
      inx
      asl pb8_ctrl
    bne pb8_loop
  next_plane:
    asl plane_modes_lo
    rol plane_modes_hi
    asl plane_modes_lo
    rol plane_modes_hi
    asl plane_counter
  bcc block_loop
  bcs add_y_to_stream_ptr
skip_whole_block:
    txa
    ;,; sec
    adc #64-1
    tax
add_y_to_stream_ptr:
  tya
  clc
  adc bagel_stream_ptr+0
  sta bagel_stream_ptr+0
  bcc no_inc_high_byte
    inc bagel_stream_ptr+1
  no_inc_high_byte:
  dec bagel_block_count
rts
.endproc

;;
; YA = low and high bytes of the input stream address
.proc bagel_bulk_upload
  sta bagel_stream_ptr+1
  sty bagel_stream_ptr+0
  txa
  pha
    ldy #0
    lda (bagel_stream_ptr), y
    sta bagel_block_count
    inc bagel_stream_ptr+0
    bne no_inc_high_byte
      inc bagel_stream_ptr+1
    no_inc_high_byte:

    upload_loop:
      ldx #64
      jsr bagel_decompress_block
      ldx #64
      block_upload_loop:
        lda ppu_buffer, x
        sta PPU_DATA
        inx
      bpl block_upload_loop
      ldx bagel_block_count
    bne upload_loop
  pla
  tax
  ldy bagel_stream_ptr+0
  lda bagel_stream_ptr+1
rts
.endproc
