.include "nes.inc"

.segment "CODE"

.proc debug_irq
  pha
    txa
    pha
      tya
      pha
        tsx
        ; load processor status and read the "B flag"
        lda $0104, x
        and #%00010000
        beq @__skip1
          jmp coredump
        @__skip1:
        ; handle other interrupts
      pla
      tay
    pla
    tax
  pla
rti
.endproc

;;
; Render game genie like CHR. Requires forced blank.
; From https://pineight.com/nes/#editor
; Trashes A, X, Y, temp+0
.proc debug_fillTestChr
  txa
  pha
    ldx #$00
    stx PPU_ADDR
    stx PPU_ADDR
    tileloop:
      ; Compute bytes making up this byte
      stx temp+0
      ldy #4
      sectionloop:
        lda #0
        lsr temp+0
        bcc @__skip1
          ora #$F0
        @__skip1:
        lsr temp+0
        bcc @__skip2
          ora #$0F
        @__skip2:
        .repeat 4
          sta PPU_DATA
        .endrepeat
        dey
      bne sectionloop
      inx
    bne tileloop
  pla
  tax
rts
.endproc

dynamic_jump:
  tya
  ; index = ((A & 0x7f) << 1) + 1
  sec
  rol
  tay
  pla
  sta temp+0
  pla
  sta temp+1
  iny
  lda (temp), y
  pha
  dey
  lda (temp), y
  pha
rts

;;
; Clears a nametable to a given tile number
;   and sets attribute values to 0.
; (Turn off rendering in PPUMASK and set the VRAM address increment
; to 1 in PPUCTRL first.)
; @param A tile number
; @param Y base address of nametable ($20, $24, $28, or $2C)
.proc clear_nametable
first_nametable_to_zero:
  ldy #$20
nametable_Y_to_zero:
  lda #$00
nametable_Y_to_A:
  pha
; Set base PPU address to YY00
    lda #$00
    jsr ppu_set_PPU_ADDR::from_AY
; Clear the 960 spaces of the main part of the nametable,
; using a 4 times unrolled loop
  pla
  ldy #$100-(960/4)
  @__loop1:
    .repeat 4
      sta PPU_DATA
    .endrepeat
    iny
  bne @__loop1
; Clear the 64 entries of the attribute table
  tya  ;,; lda #$00
  ldy #$100-64
  @__loop2:
    sta PPU_DATA
    dey
  bne @__loop2
rts
.endproc

.proc cart_detectMulticart
  lda #<RST
  cmp vectors+2
  lda #>RST
  sbc vectors+3
rts
.endproc
