; PackBits RLE unpacker
; Copyright (c) 2015-2016  Johnathan Roatch
; 
; Copying and distribution of this file, with or without
; modification, are permitted in any medium without royalty
; provided the copyright notice and this notice are preserved
; in all source code copies.
; This file is offered as-is, without any warranty.
;
; Version History:
;   2016-12-11: Changed to use registers for input address,
;               and added option to preserve X. (bytes: 46 ROM, 2 RAM)
;   2015-03-04: Initial release. (bytes: 46 ROM, 2 RAM)
;
; PackBits stream format:
;     0nnnnnnn ...    : N+1 literal bytes.
;     10000000        : End stream.
;     1nnnnnnn xx     : 129-N Run of X.
;

.importzp temp
.export pkb_decode

PPU_DATA        = $2007

; Compile time options. base amount of space: 46 ROM, 2 RAM

.ifndef PKB_PRESERVE_X
PKB_PRESERVE_X = 1  ; (bytes: +4 ROM, +1 RAM)
.endif

.segment "CODE"

;;
; Unpacks a PackBits stream to the PPU_ADDR
;
; PPU_ADDR is assumed set.
; Y: The low byte of the stream pointer.
; A: the high byte of the stream pointer.
; Trashes: X if PKB_PRESERVE_X is disabled. temp 0 ~ 2.
.proc pkb_decode
stream_ptr      = temp+0
stream_ptr_lo   = stream_ptr+0
stream_ptr_hi   = stream_ptr+1
.if ::PKB_PRESERVE_X
temp_x          = temp+2
  stx temp_x
.endif
  sta stream_ptr_hi
  ldx #$00
  stx stream_ptr_lo
  @input_loop:
    lda (stream_ptr), y
    iny
    bne @skip_hi
      inc stream_ptr_hi
    @skip_hi:
    cpx #$00
    beq @read_tag
      bpl @run_mode
      @lit_mode:
        sta PPU_DATA
        inx
      jmp @input_loop
      @run_mode:
        inx
        @run_loop:
          sta PPU_DATA
          dex
        bne @run_loop
      beq @input_loop   ; bra

    @read_tag:
      eor #$ff
      tax
    bmi @input_loop
      inx
    bpl @input_loop
  @end:
  lda stream_ptr_hi
.if ::PKB_PRESERVE_X
  ldx temp_x
.endif
rts
.endproc

